/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@itmages.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <windows.h>
#include <libintl.h>
#include <locale.h>
#include "errorshandler.h"

void ErrorsHandler(ItmagesErrorIds error_number)
{
    switch (error_number) {
    case ERROR_WRITEFILE:
        MessageBox(
                NULL,
                gettext("Failed to open the file for writing. Possibly another program already use it."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_READFILE:
        MessageBox(
                NULL,
                gettext("Failed to read from the file. Possibly another program already use it."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_CURL:
        MessageBox(
                NULL,
                gettext("Upload error. Please check your internet connection."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_MEMALLOC:
        MessageBox(
                NULL,
                gettext("Failed to allocate the memory."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_PARSERESPONSE:
        MessageBox(
                NULL,
                gettext("Failed to parse the response from the server."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_GETIMGPATH:
        MessageBox(
                NULL,
                gettext("Failed to get the image path."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_SUPPORTEDIMG:
        MessageBox(
                NULL,
                gettext("Unsupported image type. ITmages supports only jpg/jpeg, png and gif images."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_GETIMGSIZE:
        MessageBox(
                NULL,
                gettext("Failed to get the image's size."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_MAXIMGSIZE:
        MessageBox(
                NULL,
                gettext("Unsupported image size. Image's size have to be less than 6 Mb."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_OPENCLIPBOARD:
        MessageBox(
                NULL,
                gettext("Failed to open the clipboard."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_REGISTERWNDCLS:
        MessageBox(
                NULL,
                gettext("Failed to register window's class."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_GETSETTINGSDIR:
        MessageBox(
                NULL,
                gettext("Failed to get the default settings directory."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_GETCOOKIESDIR:
        MessageBox(
                NULL,
                gettext("Failed to get the default cookies directory."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_READCONFFILE:
        MessageBox(
                NULL,
                gettext("Failed to read the configuration file."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_WRITECONFFILE:
        MessageBox(
                NULL,
                gettext("Failed to write the settings into the configuration file."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_CREATETHREAD:
        MessageBox(
                NULL,
                gettext("Failed to create a new thread."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_CONNECTION:
        MessageBox(
                NULL,
                gettext("Failed to connect to the server."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_LOGIN:
        MessageBox(
                NULL,
                gettext("Failed to login. Check your login and password."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_LOGOUT:
        MessageBox(
                NULL,
                gettext("Failed to logout."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_BIGWIDTH:
        MessageBox(
                NULL,
                gettext("The reduce value have to be less than actual width of the image."),
                gettext("ITmages - Warning"),
                MB_OK | MB_ICONWARNING);
        MessageBeep(MB_ICONWARNING);
        break;
    case ERROR_BADIMGSIZE:
        MessageBox(
                NULL,
                gettext("Wrong image size. Possible, image is corrupted."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_GETIMGWIDTH:
        MessageBox(
                NULL,
                gettext("Failed to get the image's width."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    case ERROR_GETIMGINFO:
        MessageBox(
                NULL,
                gettext("Failed to get the information about the image."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
        break;
    default:
        MessageBox(
                NULL,
                gettext("Wrong error's id."),
                gettext("ITmages - Error"),
                MB_OK | MB_ICONERROR);
        MessageBeep(MB_ICONERROR);
    }
}
