/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@mail.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef IMAGEFUNC_H
#define	IMAGEFUNC_H

// Maximal image size (bytes)
#define MAX_IMAGE_SIZE 6291456

typedef struct {
    char *fullname;
    char *mime;
    size_t width;
    size_t height;
    unsigned long size;
} ItmagesImageInfo;

// Sets image's extension to the lower case
void SetExtToLower(wchar_t *ext);

// Gets rescale filter type
size_t GetFilterType(size_t filter_index);

/* Retrieves the information about the image(e.g fullname, width,
height, size etc.) and sets it to the ItmagesImageInfo structure. */
int GetImageInfo(
        wchar_t *image_path,
        wchar_t *basename,
        wchar_t *ext,
        ItmagesImageInfo *imageInfo);

/* Frees the memory used by the ItmagesImageInfo structure. */
void FreeImageInfo(ItmagesImageInfo *imageInfo);

/* Respectively resizes the image.
Returns 0 if the image was resized successfully.
Returns -1 if the image's resize was failed. */
int ResizeImageTo(
        wchar_t *image_path,
        size_t new_width,
        size_t filter_index);

/* Rotates image to the selected angle.
Returns 0 if the image was rotated successfully.
Returns -1 if the image's rotate was failed. */
int RotateImageTo(wchar_t *image_path, int angle);

/* Creates HBITMAP from the image file with selected width and height.
Returns HBITMAP of the image if the function was successfull.
Returns NULL if the function was failed. */
HBITMAP HBITMAPFromFile(
        HWND image_wnd,
        wchar_t *image_path,
        size_t max_width,
        size_t max_height);

/* Checks image's extesion.
Returns 0 if the image has an appropriate extension (*.jpg,*.jpeg,*.png,*.gif).
Returns -1 if image has inappropriate extension.*/
int CheckImageExtension(wchar_t *ext);

/* Retrieves image's size.
Returns size of the image if the image's size is less than MAX_IMAGE_SIZE.
Returns -1 if the image's size is more than MAX_IMAGE_SIZE. */
int GetImageSize(wchar_t *image_path);

/* Forms fullname of the image using a basename and extension.
Returns pointer to a string with the image's fullname. */
char *GetImageFullname(wchar_t *basename, wchar_t *ext);

#endif