!include "MUI.nsh"

; Application's version
!define VERSION "0.3"

; Application's name
!define APP_NAME "ITmages upload client"

; Default icon for the installer
!define MUI_ICON "D:\itmages.ico"

; Enables exit warning dialog
!define MUI_ABORTWARNING

; Setting application's name
name "${APP_NAME}"

; Name of the installation file
OutFile "itmages-extension-${VERSION}-win32.exe"

; Default installation directory
InstallDir "$PROGRAMFILES\ITmages"

; Compression type
SetCompressor lzma

; Variables
Var StartMenuFolder

;Request application privileges for Windows Vista and Windows 7
RequestExecutionLevel user

; Install sections
!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "D:\ITmages\docs\COPYING.txt"
!insertmacro MUI_PAGE_DIRECTORY

!define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKCU" 
!define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\ITmages" 
!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
  
!insertmacro MUI_PAGE_STARTMENU Application $StartMenuFolder
!insertmacro MUI_PAGE_INSTFILES

; Uninstall sections
!insertmacro MUI_UNPAGE_WELCOME
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

;Languages
!insertmacro MUI_LANGUAGE "Russian"
!insertmacro MUI_LANGUAGE "English"

; Associates ITmages client with the selected file extension
Function AssociateWithExtension

Exch $0

${if} $LANGUAGE = 1049
	WriteRegStr HKCR "SystemFileAssociations\$0\Shell\itmagesupload" "" "  ITmages"
${elseif} $LANGUAGE = 1033
	WriteRegStr HKCR "SystemFileAssociations\$0\Shell\itmagesupload" "" "Upload to ITmages"
${endif}

WriteRegStr HKCR "SystemFileAssociations\$0\Shell\itmagesupload" "Icon" "$INSTDIR\itmages.exe,0"
WriteRegStr HKCR "SystemFileAssociations\$0\Shell\itmagesupload\Command" "" "$INSTDIR\itmages.exe %1"

Pop $0

FunctionEnd

; Checks if the application was already installed
Function IsInstalled

ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages" "UninstallString"

StrCmp $R0 "" NotInstalled +1
${if} $LANGUAGE = 1049
	messagebox::show MB_DEFBUTTON4|MB_TOPMOST "ITmages upload client" \
		"0,103" \
		"ITmages upload client  ." \
		"" ""
${elseif} $LANGUAGE = 1033
	messagebox::show MB_DEFBUTTON4|MB_TOPMOST "ITmages upload client" \
		"0,103" \
		"ITmages upload client is already installed." \
		"Uninstall" "Cancel"
${endif}
Pop $R1
StrCmp $R1 2 Quit +1
Exec $R0
Quit:
  Quit
NotInstalled:
Call InstallLanguage

FunctionEnd

; Installer language selection function
Function InstallLanguage

Push ""
Push ${LANG_ENGLISH}
Push English
Push ${LANG_RUSSIAN}
Push 
Push A
LangDLL::LangDialog "Choose the language" "Select the language of the installer"
Pop $LANGUAGE
StrCmp $LANGUAGE "cancel" 0 +2
	Abort

FunctionEnd

;Uninstaller language selection function
Function un.UninstallLanguage

Push ""
Push ${LANG_ENGLISH}
Push English
Push ${LANG_RUSSIAN}
Push 
Push A
LangDLL::LangDialog "Choose the language" "Select the language of the uninstaller"
Pop $LANGUAGE
StrCmp $LANGUAGE "cancel" 0 +2
	Abort

FunctionEnd

Section "ITmages" SecMain

; Installation files
SetOutPath "$INSTDIR"
File "D:\ITmages\"

SetOutPath "$INSTDIR\docs"
File "D:\ITmages\docs\"

SetOutPath "$INSTDIR\locale\ru\LC_MESSAGES"
File "D:\ITmages\locale\ru\LC_MESSAGES\"

; Creating uninstaller
WriteUninstaller "$INSTDIR\uninstall.exe"

; Putting information about the application to the registry
WriteRegStr HKCU "Software\ITmages" "" $INSTDIR

; Creating startmenu entry
!insertmacro MUI_STARTMENU_WRITE_BEGIN Application

	CreateDirectory "$SMPROGRAMS\$StartMenuFolder"
	${if} $LANGUAGE = 1049
		CreateShortCut "$SMPROGRAMS\$StartMenuFolder\ITmages -    .lnk" "$INSTDIR\itmages.exe"  
	${elseif} $LANGUAGE = 1033
		CreateShortCut "$SMPROGRAMS\$StartMenuFolder\ITmages Upload Client.lnk" "$INSTDIR\itmages.exe"  
	${endif}
	CreateShortCut "$SMPROGRAMS\$StartMenuFolder\Uninstall.lnk" "$INSTDIR\Uninstall.exe"  

!insertmacro MUI_STARTMENU_WRITE_END

; Creating desktop shourtcut
${if} $LANGUAGE = 1049
	CreateShortCut "$DESKTOP\ITmages -    .lnk" "$INSTDIR\itmages.exe"
${elseif} $LANGUAGE = 1033
	CreateShortCut "$DESKTOP\ITmages Upload Client.lnk" "$INSTDIR\itmages.exe"
${endif}

; Registry strings to add uninstall information to Add/Remove programs
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages" "DisplayName" "ITmages upload client"
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages" "UninstallString" "$\"$INSTDIR\uninstall.exe$\""
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages" "DisplayIcon" "$\"$INSTDIR\itmages.exe$\""
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages" "Publisher" "ITmages"
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages" "DisplayVersion" "${VERSION}"

; Create association with png files
Push ".png"
Call AssociateWithExtension

; Create association with jpeg files
Push ".jpeg"
Call AssociateWithExtension

; Create association with jpg files
Push ".jpg"
Call AssociateWithExtension

; Create association with gif files
Push ".gif"
Call AssociateWithExtension

SectionEnd

Function .onInit 

Call IsInstalled

FunctionEnd

Section "Uninstall"
; Removing application's directory
RMDir /r "$INSTDIR"

; Removing startmenu entry
!insertmacro MUI_STARTMENU_GETFOLDER Application $StartMenuFolder    
RMDir /r "$SMPROGRAMS\$StartMenuFolder"

; Removing desktop shortcat
Delete "$DESKTOP\ITmages*.lnk"

; Removing "Upload to ITmages" action from the context menu
DeleteRegKey HKCR "SystemFileAssociations\.png\Shell\itmagesupload"
DeleteRegKey HKCR "SystemFileAssociations\.jpeg\Shell\itmagesupload"
DeleteRegKey HKCR "SystemFileAssociations\.jpg\Shell\itmagesupload"
DeleteRegKey HKCR "SystemFileAssociations\.gif\Shell\itmagesupload"

; Removing the uninstall information from the registry
DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ITmages"

; Removing information about the application from the registry
DeleteRegKey /ifempty HKCU "Software\ITmages"

SectionEnd

Function un.onInit

Call un.UninstallLanguage

FunctionEnd
