/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@itmages.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <commctrl.h>
#include <curl.h>
#include <mxml.h>
#include "itmagescmd.h"
#include "itmagesgui.h"
#include "errorshandler.h"

char g_ErrorBuffer[CURL_ERROR_SIZE];

size_t ItmagesCallbackWrite(
        char *data,
        size_t size,
        size_t nmemb,
        FILE *write_file)
{
    return fwrite(data, size, nmemb, write_file);
}

size_t ItmagesCallbackRead(
        char *data,
        size_t size,
        size_t nmemb,
        FILE *read_file)
{
    return fread(data, size, nmemb, read_file);
}

int CallbackProgressbar(
        HWND upload_progress,
        double total_down,
        double down,
        double total_up,
        double up)
{
    SendMessage(upload_progress, PBM_SETRANGE, 0, (LPARAM) MAKELONG(0, 10));
    SendMessage(upload_progress, PBM_SETPOS, (WPARAM) (up / total_up * 10), 0);
    SendMessage(upload_progress, PBM_SETSTEP, 0, 0);
    return 0;
}

int ItmagesCheckConnection(
        char *cookies_path,
        char *proxy_adrs,
        char *proxy_port)
{
    CURL *curl;
    FILE *write_file;
    CURLcode result;
    long resp_code = 0;

    write_file = fopen("check.xml", "w");
    if (write_file == NULL) {
        ErrorsHandler(ERROR_WRITEFILE);
        return -1;
    } else {
        curl = curl_easy_init();
        if (curl) {
            curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, g_ErrorBuffer);
            curl_easy_setopt(curl, CURLOPT_URL, ITMAGES_CHECK_URL);
            curl_easy_setopt(curl, CURLOPT_USERAGENT, ITMAGES_USER_AGENT);
            curl_easy_setopt(curl, CURLOPT_REFERER, ITMAGES_REFERER);
            curl_easy_setopt(curl, CURLOPT_ENCODING, "gzip,deflate");
            curl_easy_setopt(curl, CURLOPT_COOKIEJAR, cookies_path);
            curl_easy_setopt(curl, CURLOPT_COOKIEFILE, cookies_path);
            curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
            if (proxy_adrs != NULL && proxy_port != NULL) {
                size_t port_num = atoi(proxy_port);
                curl_easy_setopt(curl, CURLOPT_PROXY, proxy_adrs);
                curl_easy_setopt(curl, CURLOPT_PROXYPORT, port_num);
            }
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, ItmagesCallbackWrite);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, write_file);
            result = curl_easy_perform(curl);

            if (result != CURLE_OK) {
                curl_easy_cleanup(curl);
                fclose(write_file);
                remove("check.xml");
                ErrorsHandler(ERROR_CURL);
                return -1;
            }
            curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &resp_code);
        }
        curl_easy_cleanup(curl);
        fclose(write_file);
        remove("check.xml");
        if (resp_code == 404) {
            return 1;
        } else {
            return 0;
        }
    }
}

int ItmagesLogin(
        char* usr_login,
        char* usr_passwd,
        char *cookies_path,
        char *proxy_adrs,
        char *proxy_port)
{
    CURL *curl;
    FILE *write_file;
    CURLcode result;
    struct curl_httppost *post = NULL;
    struct curl_httppost *last = NULL;

    write_file = fopen("login.xml", "w");
    if (write_file == NULL) {
        ErrorsHandler(ERROR_WRITEFILE);
        return -1;
    } else {
        curl = curl_easy_init();
        if (curl) {
            curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, g_ErrorBuffer);
            curl_easy_setopt(curl, CURLOPT_URL, ITMAGES_LOGIN_URL);
            curl_easy_setopt(curl, CURLOPT_USERAGENT, ITMAGES_USER_AGENT);
            curl_easy_setopt(curl, CURLOPT_REFERER, ITMAGES_REFERER);
            curl_easy_setopt(curl, CURLOPT_ENCODING, "gzip,deflate");
            curl_easy_setopt(curl, CURLOPT_COOKIEJAR, cookies_path);
            curl_easy_setopt(curl, CURLOPT_COOKIEFILE, cookies_path);
            curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
            if (proxy_adrs != NULL && proxy_port != NULL) {
                size_t port_num = atoi(proxy_port);
                curl_easy_setopt(curl, CURLOPT_PROXY, proxy_adrs);
                curl_easy_setopt(curl, CURLOPT_PROXYPORT, port_num);
            }
            curl_easy_setopt(curl, CURLOPT_POST, 1);
            curl_formadd(
                    &post, &last,
                    CURLFORM_COPYNAME, "name",
                    CURLFORM_COPYCONTENTS, usr_login,
                    CURLFORM_END);
            curl_formadd(
                    &post, &last,
                    CURLFORM_COPYNAME, "passwd",
                    CURLFORM_COPYCONTENTS, usr_passwd,
                    CURLFORM_END);
            curl_easy_setopt(curl, CURLOPT_HTTPPOST, post);
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, ItmagesCallbackWrite);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, write_file);
            result = curl_easy_perform(curl);
        }
        curl_easy_cleanup(curl);
        if (result != CURLE_OK) {
            fclose(write_file);
            remove("login.xml");
            ErrorsHandler(ERROR_CURL);
            return -1;
        }
        fclose(write_file);
    }

    FILE *read_file;
    char *status;

    read_file = fopen("login.xml", "r");
    if (read_file == NULL) {
        ErrorsHandler(ERROR_READFILE);
        return -1;
    } else {
        mxml_node_t *response_tree;
        response_tree = mxmlLoadFile(NULL, read_file, MXML_OPAQUE_CALLBACK);
        fclose(read_file);
        remove("login.xml");
        mxml_node_t *response_node;
        response_node = mxmlFindElement(response_tree, response_tree,
                "response", NULL, NULL, MXML_DESCEND);
        if (response_node == NULL) {
            ErrorsHandler(ERROR_PARSERESPONSE);
            return -1;
        }
        status = response_node->child->child->value.opaque;
        if (memcmp(status, "ok", 3) == 0) {
            mxmlDelete(response_tree);
            return 0;
        } else {
            mxmlDelete(response_tree);
            return 1;
        }
    }
}

int ItmagesLogout(
        char *cookies_path,
        char *proxy_adrs,
        char *proxy_port)
{
    CURL *curl;
    CURLcode result;
    FILE *write_file;
    struct curl_httppost *post = NULL;
    struct curl_httppost *last = NULL;

    write_file = fopen("logout.xml", "w");
    if (write_file == NULL) {
        ErrorsHandler(ERROR_WRITEFILE);
        return -1;
    } else {
        curl = curl_easy_init();
        if (curl) {
            curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, g_ErrorBuffer);
            curl_easy_setopt(curl, CURLOPT_URL, ITMAGES_LOGOUT_URL);
            curl_easy_setopt(curl, CURLOPT_USERAGENT, ITMAGES_USER_AGENT);
            curl_easy_setopt(curl, CURLOPT_REFERER, ITMAGES_REFERER);
            curl_easy_setopt(curl, CURLOPT_ENCODING, "gzip,deflate");
            curl_easy_setopt(curl, CURLOPT_COOKIEFILE, cookies_path);
            curl_easy_setopt(curl, CURLOPT_COOKIEJAR, cookies_path);
            curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
            if (proxy_adrs != NULL && proxy_port != NULL) {
                size_t port_num = atoi(proxy_port);
                curl_easy_setopt(curl, CURLOPT_PROXY, proxy_adrs);
                curl_easy_setopt(curl, CURLOPT_PROXYPORT, port_num);
            }
            curl_easy_setopt(curl, CURLOPT_POST, 1);
            curl_formadd(
                    &post, &last,
                    CURLFORM_COPYNAME, "submit",
                    CURLFORM_COPYCONTENTS, "Logout",
                    CURLFORM_END);
            curl_easy_setopt(curl, CURLOPT_HTTPPOST, post);
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, ItmagesCallbackWrite);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, write_file);
            result = curl_easy_perform(curl);
        }
        curl_easy_cleanup(curl);
        if (result != CURLE_OK) {
            fclose(write_file);
            remove("logout.xml");
            ErrorsHandler(ERROR_CURL);
            return -1;
        }
        fclose(write_file);
    }

    FILE *read_file;
    char *status;

    read_file = fopen("logout.xml", "r");
    if (read_file == NULL) {
        ErrorsHandler(ERROR_READFILE);
        return -1;
    } else {
        mxml_node_t *response_tree;
        response_tree = mxmlLoadFile(NULL, read_file, MXML_OPAQUE_CALLBACK);
        fclose(read_file);
        remove("logout.xml");
        mxml_node_t *response_node;
        response_node = mxmlFindElement(response_tree, response_tree,
                "response", NULL, NULL, MXML_DESCEND);
        if (response_node == NULL) {
            ErrorsHandler(ERROR_PARSERESPONSE);
            return -1;
        }
        status = response_node->child->child->value.opaque;
        if (memcmp(status, "ok", 3) == 0) {
            mxmlDelete(response_tree);
            return 0;
        } else {
            mxmlDelete(response_tree);
            return 1;
        }
    }
}

int ItmagesUpload(
        HWND upload_progress,
        wchar_t *image_path,
        char *name,
        char *mime,
        char *cookies_path,
        char *proxy_adrs,
        char *proxy_port,
        size_t size,
        ItmagesImageLinks *imageLinks)
{
    CURL *curl;
    CURLcode result;
    FILE *write_file, *read_file;
    struct curl_httppost *post = NULL;
    struct curl_httppost *last = NULL;

    write_file = fopen("upload.xml", "w");
    if (write_file == NULL) {
        ErrorsHandler(ERROR_WRITEFILE);
        return -1;
    } else {
        read_file = _wfopen(image_path, L"rb");
        if (read_file == NULL) {
            ErrorsHandler(ERROR_READFILE);
            return -1;
        } else {
            curl = curl_easy_init();
            if (curl) {
                curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, g_ErrorBuffer);
                curl_easy_setopt(curl, CURLOPT_URL, ITMAGES_ADD_URL);
                curl_easy_setopt(curl, CURLOPT_USERAGENT, ITMAGES_USER_AGENT);
                curl_easy_setopt(curl, CURLOPT_REFERER, ITMAGES_REFERER);
                curl_easy_setopt(curl, CURLOPT_ENCODING, "gzip, default");
                curl_easy_setopt(curl, CURLOPT_COOKIEJAR, cookies_path);
                curl_easy_setopt(curl, CURLOPT_COOKIEFILE, cookies_path);
                curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
                if (proxy_adrs != NULL && proxy_port != NULL) {
                    size_t port_num = atoi(proxy_port);
                    curl_easy_setopt(curl, CURLOPT_PROXY, proxy_adrs);
                    curl_easy_setopt(curl, CURLOPT_PROXYPORT, port_num);
                }
                curl_easy_setopt(curl, CURLOPT_POST, 1);
                curl_formadd(
                        &post, &last,
                        CURLFORM_COPYNAME, "UFileManager[picture]",
                        CURLFORM_FILENAME, name,
                        CURLFORM_STREAM, read_file,
                        CURLFORM_CONTENTSLENGTH, size,
                        CURLFORM_CONTENTTYPE, mime,
                        CURLFORM_END);
                curl_formadd(
                        &post, &last,
                        CURLFORM_COPYNAME, "UFileManager[picture]",
                        CURLFORM_COPYCONTENTS, "",
                        CURLFORM_END);
                curl_easy_setopt(curl, CURLOPT_HTTPPOST, post);
                curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, ItmagesCallbackWrite);
                curl_easy_setopt(curl, CURLOPT_WRITEDATA, write_file);
                curl_easy_setopt(curl, CURLOPT_READFUNCTION, ItmagesCallbackRead);
                curl_easy_setopt(curl, CURLOPT_READDATA, read_file);
                curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
                curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, CallbackProgressbar);
                curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, upload_progress);
                result = curl_easy_perform(curl);
            }
            curl_easy_cleanup(curl);
            if (result != CURLE_OK) {
                fclose(write_file);
                fclose(read_file);
                remove("upload.xml");
                ErrorsHandler(ERROR_CURL);
                return -1;
            }
            fclose(write_file);
            fclose(read_file);
        }

        FILE *read_file;
		size_t node_len = 0;

        read_file = fopen("upload.xml", "r");
        if (read_file == NULL) {
            ErrorsHandler(ERROR_READFILE);
            return -1;
        } else {
            mxml_node_t *response_tree;
            response_tree = mxmlLoadFile(NULL, read_file, MXML_OPAQUE_CALLBACK);
            fclose(read_file);
            remove("upload.xml");
            mxml_node_t *response_node;
            response_node = mxmlFindElement(response_tree, response_tree,
                    "obj", NULL, NULL, MXML_DESCEND);
            if (response_node == NULL) {
                ErrorsHandler(ERROR_PARSERESPONSE);
                return -1;
            }
            response_node = mxmlFindElement(response_node, response_tree,
                    "full", NULL, NULL, MXML_DESCEND);
            if (response_node == NULL) {
                ErrorsHandler(ERROR_PARSERESPONSE);
                return -1;
            }

            node_len = strlen(response_node->child->value.opaque);
            imageLinks->full = (char*) malloc((node_len + 1) * sizeof(char));
            if (imageLinks->full == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }
            memcpy(imageLinks->full, response_node->child->value.opaque, node_len + 1);

            node_len = strlen(response_node->next->child->value.opaque);
            imageLinks->thumb = (char*) malloc((node_len + 1) * sizeof(char));
            if (imageLinks->thumb == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }
            memcpy(imageLinks->thumb, response_node->next->child->value.opaque, node_len + 1);

            response_node = mxmlFindElement(response_node, response_tree,
                    "id", NULL, NULL, MXML_DESCEND);
            if (response_node == NULL) {
                ErrorsHandler(ERROR_PARSERESPONSE);
                return -1;
            }

            node_len = strlen(response_node->child->value.opaque);
            imageLinks->id = (char*) malloc((node_len + 1) * sizeof(char));
            if (imageLinks->id == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }
            memcpy(imageLinks->id, response_node->child->value.opaque, node_len + 1);

            node_len = strlen(response_node->next->child->value.opaque);
            imageLinks->server = (char*) malloc((node_len + 1) * sizeof(char));
            if (imageLinks->server == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }
            memcpy(imageLinks->server, response_node->next->child->value.opaque, node_len + 1);

            node_len = strlen(response_node->next->next->child->value.opaque);
            imageLinks->key = (char*) malloc((node_len + 1) * sizeof(char));
            if (imageLinks->key == NULL) {
                ErrorsHandler(ERROR_MEMALLOC);
                exit(EXIT_FAILURE);
            }
            memcpy(imageLinks->key, response_node->next->next->child->value.opaque, node_len + 1);

            mxmlDelete(response_tree);
            return 0;
        }
    }
}

void FreeImageLinks(ItmagesImageLinks *imageLinks)
{
    if (imageLinks->full != NULL) {
        free(imageLinks->full);
        imageLinks->full = NULL;
    }
    if (imageLinks->thumb != NULL) {
        free(imageLinks->thumb);
        imageLinks->thumb = NULL;
    }
    if (imageLinks->id != NULL) {
        free(imageLinks->id);
        imageLinks->id = NULL;
    }
    if (imageLinks->server != NULL) {
        free(imageLinks->server);
        imageLinks->server = NULL;
    }
    if (imageLinks->key != NULL) {
        free(imageLinks->key);
        imageLinks->key = NULL;
    }
}
