/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@itmages.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef LIBITMAGES_H
#define	LIBITMAGES_H

#define ITMAGES_LOGIN_URL "http://www.itmages.ru/api/login"
#define ITMAGES_LOGOUT_URL "http://www.itmages.ru/api/logout"
#define ITMAGES_ADD_URL "http://www.itmages.ru/api/add"
#define ITMAGES_CHECK_URL "http://www.itmages.ru"
#define ITMAGES_REFERER "http://www.itmages.ru"
#define ITMAGES_USER_AGENT "Itmages upload client"

typedef struct {
    char *full;
    char *thumb;
    char *id;
    char *server;
    char *key;
} ItmagesImageLinks;

/* Checks connection with the ITmages server.
Returns 0 if the ITmages server is available.
Returns 1 if the ITmages server is not available.
Returns -1 if there were some errors during the connection checking. */
int ItmagesCheckConnection(
        char *cookies_path, // Path to the cookies' file
        char *proxy_adrs,
        char *proxy_port);

/* Logins to the ITmages account.
Returns 0 if the login was successfull.
Returns 1 if the login was failed.
Returns -1 if there were some errors during the login. */
int ItmagesLogin(
        char* usr_login,
        char* usr_passwd,
        char *cookies_path, // Path to the cookies' file
        char *proxy_adrs,
        char *proxy_port);

/* Logouts from the ITmages account.
Returns 0 if the logout was successfull.
Returns 1 if the logout was failed.
Returns -1 if there were some errors during the logout.*/
int ItmagesLogout(
        char *cookies_path,
        char *proxy_adrs,
        char *proxy_port);

/* Uploads selected image to the ITmages server.
Returns 0 if the image was uploaded successfully.
Returns -1 if there were some error during the uplaod. */
int ItmagesUpload(
        HWND upload_progress, // Handle of the upload progressbar
        wchar_t *image_path, // Path to the upload image
        char *name, // fullname of the image (e.g. "image.png")
        char *mime, // mime-type of the image(e.g "image/png")
        char *cookies_path, // Path to the cookies' file
        char *proxy_adrs,
        char *proxy_port,
        size_t size,
        ItmagesImageLinks *imageLinks);

// Frees the memory used by the ItmagesImageLinks structure.
void FreeImageLinks(ItmagesImageLinks *imageLinks);

#endif

