/*
    ITmages  upload client for Windows
    Copyright (C) 2011 Dmitriy Simbiriatin <slpiv@itmages.ru>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef ITMAGESCLIENT_H
#define	ITMAGESCLIENT_H

// Windows' sizes and positions
#include "sizes.h"

// Default application's icon
#define ID_APPICON 101

// Custom messages
#define WM_FINISHTHREAD WM_APP+ 1

#define MAX_LOGIN 32
#define MAX_PASSWD 32
#define MAX_ADDRSS 15
#define MAX_PORT 4
#define MAX_DIM_CHARS 4
#define MAX_STATE 5
#define MAX_INDEX 1
#define MAX_INFO 1000
#define MAX_PREVIEW 200

#define ITMAGES_DIR L"\\ITmages"
#define CONFIG_FILE L"\\config.ini"
#define COOKIES_FILE L"\\itmages.txt"
#define ITMAGES_REG_KEY "Software\\ITmages"
#define DOCS_DIR "\\docs\\"
#define LOCALE_DIR "\\locale"

typedef enum {
    ID_LOGIN_TABCONTROL = 1,
    ID_LOGIN = 2,
    ID_LOGIN_TXT = 3,
    ID_PASSWD = 4,
    ID_PASSWD_TXT = 5,
    ID_CHECK_WITHOUTLOGIN = 6,
    ID_CHECK_SAVEDATA = 7,
    ID_CHECK_RESIZEIMG = 8,
    ID_RESIZEIMG_SIZE = 9,
    ID_RESIZEIMGSIZE_UPDOWN = 10,
    ID_RESIZE_FILTER = 11,
    ID_CHECK_ROTATEIMG = 12,
    ID_ROTATEIMG_ANGLE = 13,
    ID_CHECK_USEPROXY = 14,
    ID_PROXY_ADDRESS = 15,
    ID_PROXYADDRESS_TXT = 16,
    ID_PROXY_PORT = 17,
    ID_PROXYPORT_TXT = 18,
    ID_ABOUT_BUTTON = 19,
    ID_UPLOAD_BUTTON = 20,
    ID_UPLOAD_PROGRESSBAR = 21,
    ID_IMAGE_PREVIEW = 22,
    ID_IMAGE_INFO = 23,
    ID_LINKS_LIST = 24,
    ID_LINK_PREVIEW = 25,
    ID_COPYTOCLIPBOARD = 26,
    ID_OPENINBROWSER = 27,
    ID_ABOUT_TABCONTROL = 28,
    ID_ITMAGES_IMG = 29,
    ID_ABOUT_INFO = 30,
    ID_LICENSE_INFO = 31,
    ID_CREDITS_INFO = 32,
    ID_CLOSE_BUTTON = 33
} ItmagesWindowIds;

typedef struct {
    char *login;
    char *passwd;
    char *address;
    char *port;
} ItmagesUserInfo;

typedef struct {
    wchar_t *config;
    wchar_t *cookies;
} ItmagesConfig;

typedef struct {
    wchar_t *path;
    wchar_t *basename;
    wchar_t *ext;
} ItmagesImageData;

// Login window ----------------------------------------------------------------
INT APIENTRY WinMain(
        HINSTANCE inst,
        HINSTANCE prev_inst,
        LPSTR cmd_line,
        int cmd_show);
LRESULT CALLBACK WinMainProc(
        HWND login_wnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam);

INT APIENTRY WinLogin();
void CreateLoginWnd();
void CreateLoginTabControl();
// -----------------------------------------------------------------------------

// About window ----------------------------------------------------------------
LRESULT CALLBACK WinAboutProc(
        HWND about_wnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam);
INT APIENTRY WinAbout();
void CreateAboutWnd();
void CreateAboutTabControl();
// -----------------------------------------------------------------------------

// Ready window ----------------------------------------------------------------
LRESULT CALLBACK WinReadyProc(
        HWND ready_wnd,
        UINT message,
        WPARAM wParam,
        LPARAM lParam);
INT APIENTRY WinReady();
void CreateReadyWnd();
// -----------------------------------------------------------------------------

// Upload Threads --------------------------------------------------------------
UINT APIENTRY UploadWithLoginThread(HWND login_wnd);
UINT APIENTRY UploadWithoutLoginThread(HWND login_wnd);
// -----------------------------------------------------------------------------

/* Wordaround to fix the background for static controls and --------------------
checkboxes with the enabled visual style. */
void UpdateBackgroundBrush(HWND wnd);
void SetThemedBackground(
        HWND wnd,
        HWND static_wnd,
        HDC static_hdc);
// -----------------------------------------------------------------------------

/* Creates an Open file dialog to choose the image to upload.
Returns a pointer to a string that contains the path to the image.*/
char *CreateOpenImageDialog();

// Creates tooltip with the selected text.
void CreateTooltip(
        HWND parent_wnd,
        HWND tooltip_wnd,
        char *tooltip_txt);

/* Retrives the system's default dirs for the application's
settings and cookies' files. */
void GetConfigFiles(ItmagesConfig *configFiles);

// Frees the memory used by the ItmagesConfig structure.
void FreeConfig(ItmagesConfig *configFiles);

/* Restrievs the application's installation path from the registry.
Returns a pointer to a string that contains the client's installation
path if the function was successfull.
Returns NULL if the function was failed.*/
char *GetInstallDir();

/* Forms a path to the locale directory.
Returns a pointer to a string that contains the client's locale directory
if the function was successfull.
Returns NULL if the function was failed. */
char *GetLocaleDir();

/* Loads txt files from the "docs" directory such as COPYING.txt, CREDITS.txt
Returns 0 if the file loading was successfull.
Returns -1 if the function was failed. */
int LoadDocFile(
        HWND load_wnd,
        char *docs_dir,
        char *load_file);

// Shows the selected link type inside the preview.
void ShowSelectedLink(
        HWND link_preview,
        HWND bsr_button,
        char *preview_buffer,
        size_t index);

// Sets the default system font for the choosen range of the child windows.
void SetSystemFont(
        HWND parent_wnd,
        ItmagesWindowIds first_wnd,
        ItmagesWindowIds last_wnd);

// Hides the selected range of the child windows.
void HideWindows(
        HWND parent_wnd,
        ItmagesWindowIds first_wnd,
        ItmagesWindowIds last_wnd);

// Shows the selected range of the tab's windows and hides another ones/
void ShowTabWindows(
        HWND parent_wnd,
        ItmagesWindowIds first_wnd,
        ItmagesWindowIds last_wnd,
        ItmagesWindowIds first_show,
        ItmagesWindowIds last_show);

// Frees the memory used by the ItmagesUserInfo structure.
void FreeUserInfo(ItmagesUserInfo *userInfo);

// Frees the memory used by the ItmagesImageData structure.
void FreeImageData(ItmagesImageData *imageData);

#endif